#include <stdio.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <sys/ioctl.h>
#include "usbdevice_fs.h"

static void
usage(void)
{
	printf("Usage: usbcon -c|-d [-i ifnum] /proc/bus/usb/NNN/MMM\n");
	printf("\n");
	printf("Purpose of this program is to connect or disconnect USB device from its kernel\n");
	printf("driver.  -c connects device to the kernel driver, -d makes it available for\n");
	printf("usbfs.\n");
}

int
main(int argc, char* argv[])
{
	int fd;
	struct usbdevfs_ioctl io;
	int r;
	int c;
	
	io.ioctl_code = -1;
	io.ifno = 0;
	while ((c = getopt(argc, argv, "cdhi:")) != -1) {
		switch (c) {
			case 'c':
				io.ioctl_code = USBDEVFS_CONNECT;
				break;
			case 'd':
				io.ioctl_code = USBDEVFS_DISCONNECT;
				break;
			case 'h':
				usage();
				return 1;
			case 'i':
				io.ifno = strtol(optarg, NULL, 0);
				break;
			case '?':
				usage();
				return 2;
		}
	}
	if (io.ioctl_code == -1) {
		fprintf(stderr, "Either -c or -d must be specified\n\n");
		usage();
		return 97;
	}
	if (optind >= argc) {
		fprintf(stderr, "Missing USB device filename\n\n");
		usage();
		return 98;
	}
	
	fd = open(argv[optind], O_RDWR);
	if (fd == -1) {
		fprintf(stderr, "Cannot open %s: %s\n", argv[optind], strerror(errno));
		return 99;
	}
	r = ioctl(fd, USBDEVFS_IOCTL, &io);
	if (r < 0) {
		perror("ioctl failed");
	} else if (r) {
		fprintf(stderr, "Unexpected ioctl return: %d\n", r);
	} else {
		printf("OK\n");
	}
	close(fd);
	return 0;
}
